# -*- coding: utf-8 -*-
import logging
logger = logging.getLogger(__name__)


def initialize(context):
    try:
        from plone.app.theming.transform import ThemeTransform
        from perfmetrics import Metric
        from zperfmetrics import ZMetric
    except ImportError:
        logger.info('No Plone patches for zperfmetrics')
        return

    # patch to measure plone.app.theming
    logger.info('Activating Plone patches for zperfmetrics')
    ThemeTransform.setupTransform = Metric(
        stat='diazo.setup',
        method=True,
    )(
        ThemeTransform.setupTransform
    )
    ThemeTransform.transformIterable = ZMetric(
        stat='diazo.transform',
        method=True
    )(
        ThemeTransform.transformIterable
    )



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
import os

version = '1.0a0'
shortdesc = 'Perfmetrics configuration for Zope/Plone.'
longdesc = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
longdesc += open(os.path.join(os.path.dirname(__file__), 'CHANGES.rst')).read()
longdesc += open(os.path.join(os.path.dirname(__file__), 'LICENSE.rst')).read()

setup(
    name='zperfmetrics',
    version=version,
    description=shortdesc,
    long_description=longdesc,
    classifiers=[
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python',
        'Topic :: Software Development',
    ],
    keywords='plone zope perfmetrics statsd graphite monitor performance',
    author='BlueDynamics Alliance',
    author_email='dev@bluedynamics.com',
    url="http://github.com/collective/zperfmetrics",
    license='Simplified BSD',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    zip_safe=True,
    install_requires=[
        'perfmetrics',
        'setuptools',
        'zope.globalrequest',
        'Zope2',
    ],
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: utf-8 -*-
from perfmetrics import Metric
from perfmetrics import client_stack
from perfmetrics import statsd_client_stack
from time import time
from zope.globalrequest import getRequest

import functools
import zperfmetrics.patches

zperfmetrics.patches  # flake 8 happiness


class ZMetric(Metric):
    """Make metric decorator/context managers for Zope Requests.
    """

    def __call__(self, f):
        """Decorate a function or method so it can send statistics to statsd.
        """
        func_name = f.__name__
        func_full_name = (f.__module__, func_name)

        instance_stat = self.stat
        rate = self.rate
        method = self.method
        count = self.count
        timing = self.timing
        timing_format = self.timing_format
        instance_random = self.random

        def call_with_metric(*args, **kw):
            if rate < 1 and instance_random() >= rate:
                # Ignore this sample.
                return f(*args, **kw)

            stack = statsd_client_stack.stack
            client = stack[-1] if stack else client_stack.default
            if client is None:
                # No statsd client has been configured.
                return f(*args, **kw)

            if instance_stat:
                statpart = instance_stat
            elif method:
                cls = args[0].__class__
                statpart = '.'.join([cls.__module__, cls.__name__, func_name])
            else:
                statpart = func_full_name

            stat = ''
            request = getRequest()
            if request:
                stat += '.PATH.' + request['PATH_INFO']
                stat += '.' + statpart
            else:
                stat = statpart

            if timing:
                if count:
                    buf = []
                    client.incr(stat, 1, rate, buf=buf, rate_applied=True)
                else:
                    buf = None
                start = time()
                try:
                    return f(*args, **kw)
                finally:
                    elapsed_ms = int((time() - start) * 1000.0)
                    client.timing(timing_format % stat, elapsed_ms,
                                  rate, buf=buf, rate_applied=True)
                    if buf:
                        client.sendbuf(buf)

            else:
                if count:
                    client.incr(stat, 1, rate, rate_applied=True)
                return f(*args, **kw)

        return functools.update_wrapper(call_with_metric, f)

    # Metric can also be used as a context manager.

    def __exit__(self, _typ, _value, _tb):
        rate = self.rate
        if rate < 1 and self.random() >= rate:
            # Ignore this sample.
            return

        stack = statsd_client_stack.stack
        client = stack[-1] if stack else client_stack.default
        if client is not None and self.stat:
            buf = []
            request = getRequest()
            if request:
                stat = '.PATH.' + request['PATH_INFO']
                stat += '.' + self.stat
            else:
                stat = self.stat
            if stat:
                if self.count:
                    client.incr(stat, rate=rate, buf=buf, rate_applied=True)
                if self.timing:
                    elapsed = int((time() - self.start) * 1000.0)
                    client.timing(self.timing_format % stat, elapsed,
                                  rate=rate, buf=buf, rate_applied=True)
                if buf:
                    client.sendbuf(buf)

zmetric = ZMetric()

zmetricmethod = ZMetric(method=True)



# -*- coding: utf-8 -*-
from perfmetrics import set_statsd_client

import socket
import urlparse


class PerfmetricsConfig(object):

    def __init__(self, config):
        prefix = ''
        if config.before:
            prefix += config.before + '.'
        if config.hostname:
            prefix += socket.gethostname() + '.'
        if config.after:
            prefix += config.after + '.'

        url = urlparse.urlparse(config.uri)
        if url.query:
            qs = urlparse.parse_qs(url.query)
            preset = qs.get('prefix', [])
            if preset:
                prefix = preset[0] + '.' + prefix
        parts = list(url)
        parts[4] = 'prefix=' + prefix
        set_statsd_client(urlparse.urlunparse(parts))

    def prepare(self, *args):
        pass

    def create(self, *args):
        pass
